#include <userint.h>
#include <ansi_c.h>           
#define  WIENVME
#define USE_DAQ



// Izberi ustrezni interface v meniju projektnega okna
//  Options->CompilerDefines (dodaj /DSISVME ali /DWIENVME)


#ifdef USE_DAQ
#  define USE_CAMAC
#  ifdef SISVME
#    include "sisvme_dll.h"
#  endif
#  ifdef WIENVME
#    include "wienvme_dll.h"
#  endif
#endif


#include "CAENV462.h"   
#include "CAENV965_DEF.h"
#include "mdaq.h"




#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <signal.h>
#include <utility.h>

#define MAXCH 16
#define MAXSIZE 0x1FFFF
typedef struct {
int recid;
int size;
int time;
int id;
int nbins;
char name[120];
float xmin;
float xmax;
float data[MAXSIZE];
} TH1F;

TH1F tch[2*MAXCH];

int TH1Finit(int id, char *hname, int nbins, float xmin, float xmax){
  tch[id].recid= 11;
  tch[id].id=id;
  tch[id].time=time(NULL);
  if (nbins>MAXSIZE) nbins=MAXSIZE;
  sprintf(tch[id].name,"%s",hname);
  tch[id].nbins= nbins; 
  tch[id].xmin = xmin;
  tch[id].xmax = xmax;
  tch[id].size = sizeof(TH1F) - (MAXSIZE - nbins)* sizeof(float);
  return 0;
}

/* VME modules */
#define V462_ADDR 0x100300
#define CAEN_V965_0 0x110000
#define CAEN_V965_1 0x550000
unsigned long addr[2]={CAEN_V965_0, CAEN_V965_1};
int nv965=1;

#define NCH_ADC   16  
#define MAXCH_ADC 0x1000


#define CAEN_V965_PEDESTAL 180   // use it without the amplifier

int timer_out;

int init(int ped, int mask ){
  int i,module;
  unsigned short geo, fw, ah, al;
     
  unsigned short dum16;
  float delay=1.1e-6;
  
  V462_map (0, V462_ADDR,1);
	
  V462_set1 (0, (int)(delay*1e7));
    
  for ( module=0;module<nv965;module++){ 
    VME_A24D16_R(addr[module] + CAENV965_FW, &fw);
    geo = 0;
    VME_A24D16_W(addr[module] + CAENV965_CRN, &geo);
    //  enable channels
    for (i=0;i<32;i++)
      VME_A24D16_W(addr[module] + CAENV965_THM + 2*i, &geo);
    
    // disable channels
    geo=(1<<8);
    for (i=0;i<32;i++)
    if ((1<<i/2)&(~mask)) {
      printf("Masking channel %d\n", i/2);
      VME_A24D16_W(addr[module] + CAENV965_THM + 2*i, &geo);
    } else {
      printf("Channel %d enabled  mask=0x%04x \n", i/2, ~mask);   
    }
	//
    geo = module;
    VME_A24D16_W(addr[module] + CAENV965_GEO, &geo);
    VME_A24D16_R(addr[module] + CAENV965_GEO, &geo);
    VME_A24D16_R(addr[module] + CAENV965_ADH, &ah);
    VME_A24D16_R(addr[module] + CAENV965_ADL, &al);
  
    printf(" V965_map firmware = %d.%d\n", (fw>>8)&0xff, fw&0xff);
    printf(" V965_map geo      = %d\n", geo&0x1f );
    printf(" V965_map addr	 = 0x%04X0000\n", (ah<<8)|(al&0xff));
 	
    //----------------------------   
    dum16= 0x80;
    VME_A24D16_W(addr[module] + CAENV965_BS1, &dum16);
    VME_A24D16_W(addr[module] + CAENV965_BC1, &dum16);

    //dum16= CAEN_V965_PEDESTAL;
    dum16=ped;
    VME_A24D16_W(addr[module] + CAENV965_PED, &dum16);

    dum16= 0x5000;
    VME_A24D16_W(addr[module] + CAENV965_BS2, &dum16);

    dum16= 0x4;
    VME_A24D16_W(addr[module] + CAENV965_BS2, &dum16);
    VME_A24D16_W(addr[module] + CAENV965_BC2, &dum16);
    VME_A24D16_W(addr[module] + CAENV965_ECR, &dum16);
  }
      
      
      printf("daq::init()\n");
      return 0;
}


int clear(){
	unsigned short dum;
	dum= 0x4;
      VME_A24D16_W(CAEN_V965_0  + CAENV965_BS2, &dum);
      VME_A24D16_W(CAEN_V965_1  + CAENV965_BS2, &dum);
      VME_A24D16_W(CAEN_V965_0  + CAENV965_BC2, &dum);
      VME_A24D16_W(CAEN_V965_1  + CAENV965_BC2, &dum);
   return 0;
}

int module_header(int recid,unsigned long *data,int len){
   data[0] = recid;
   data[1] = (len >0)? len : 0 ;
   data[1] =  (data[1] +2)*sizeof(unsigned long) ;
   if (len>0) return len+2; else return 2;
}


#define MAXITER 100000
int event(unsigned long *data, int maxn){
   int edge, rg, val, ch;
   int ndata, module;
   int count=0;
   unsigned int dum,q,idx;
   unsigned long dum32,ierr;
   unsigned short *tdc16 = (unsigned short *) &dum32;  
   clear();    
   // Dual Gate generator trigger
   V462_start1 (0);

  // CAEN V965
 
   for ( module=0;module<nv965;module++){
     idx=0;
     do {
        VME_A24D16_R(addr[module] + CAENV965_SR1, &dum);
        idx++; 
     } while (!(dum&0x1) && idx<MAXITER);
	 if (idx==MAXITER) return -3-module;
     ndata=0;
     do {
      idx=count+2+ndata; 
      if (idx<maxn) VME_A24D32_R( addr[module] + CAENV965_OB, &data[idx]);  
      else return -5-module;
       
       ierr=(data[count+2+ndata]>>25)&0x3;

	   if (ierr==0) {
		     	    ch=(data[count+2+ndata]>>17)&0xf;
		     	    rg=(data[count+2+ndata]>>16)&0x1;
		     	    val=data[count+2+ndata]&0xfff;
		     	   //printf("V965 %d [%d] rg=%d %d\n",ndata,ch, rg, val);
		     	    tch[ch+rg*MAXCH].data[val]++;
	   }
	   if (ierr<3) ndata++;
	   
     } while (ierr<2);
     
     count+=module_header(addr[module],&data[count],ndata);
  }
  

  
   return count*sizeof(unsigned long);
}



#define MAX_BSIZE 10000
int main(int argc, char ** argv){
  time_t timet;
  long mask=0xFFFF;
  int nb,i, neve=10000;
  char fname[120]="tmp.dat", hname[120];
  unsigned long data[MAX_BSIZE];
  int pedestal=250;
  int hdr[4];
  FILE *fp ;

  SetStdioPort (HOST_SYSTEM_STDIO);                 
  printf("%s neve fname mask\n", argv[0]);
  if (argc>1) neve= atoi(argv[1]);
  if (argc>2) sprintf(fname,"%s" ,argv[2]);
  if (argc>3) mask= strtol(argv[3],NULL,0);
  if (argc>4) pedestal= atoi(argv[4]);  
  printf("mask=0x%x\n", mask); 
  fp= fopen(fname,"ab");
  
  // initialize VME interface	
  VME_START(NULL);  
  
  
   for (i=0;i<MAXCH;i++) if ( (0x1<<i) & mask ) {
  	  sprintf(hname,"ch0%02d", i);      
      TH1Finit(i,hname,4096,-0.5, 4095.5); 
      sprintf(hname,"ch1%02d", i);      
      TH1Finit(i+MAXCH,hname,4096,-0.5, 4095.5);
   }
   
   init(pedestal,mask);
  
   // begin run record
   hdr[0]= 1;
   hdr[1]= 4*sizeof(int);
   hdr[2]= time(NULL);
   hdr[3]= neve;

   //printf("Event %d %d %d %d  nb=%d %d\n",hdr[0],hdr[1],hdr[2],hdr[3],nb,nb/4); 

   fwrite(hdr,   sizeof(int),4 , fp);
   
   
   for (i=0;i<neve;i++){
  	 nb = event(data,MAX_BSIZE);
  	
     if (nb<=MAX_BSIZE&& (nb>0) ) {
       fwrite(data,nb,1,fp); 
       printf("[%4d] nb=%d %d\n",i,nb, data[1]);
     } else printf("nb=%d MAX_BSIZE=%d\n",nb, MAX_BSIZE);   
  
   }
  
   for (i=0;i<MAXCH;i++) if ( (0x1<<i) & mask ) {
    printf("channel %d size=%d\n",i,tch[i].size);
    fwrite( &tch[i],tch[i].size,1, fp);
    fwrite( &tch[i+MAXCH],tch[i+MAXCH].size,1, fp);     
   }

   // end run record
   hdr[0]= 2;
   hdr[1]= 4*sizeof(int);
   hdr[2]= time(NULL);
   hdr[3]= neve;

   //printf("Event %d %d %d %d  nb=%d %d\n",hdr[0],hdr[1],hdr[2],hdr[3],nb,nb/4); 

   fwrite(hdr,   sizeof(int),4 , fp);
  
   
 
  fclose(fp);
   		 // close VME interface
  VME_STOP(); 
  
  return 0;
}
