/* 
**	memlog - memory status logger
**
**	File:    memlog.c
**	Author:  John Fitzgibbon (fitz@jfitz.com http://www.jfitz.com)
**	Version: 1.0
**	Date:    11/08/1999
**
**	This program and code is FREEWARE.
**
**  Command line can include one parameter - the number of milliseconds between
**  each memory check. (Default is 5000).
**
**  Output format is:
**  Date,Time,Free Physical,Free Page,Free Virtual,Total Physical,Total Page,Total Virtual
**
**  All memory figures are in bytes.
**
**  Revision History
**  11/08/1999 1.0 Original Version 
**  05/14/2003 1.1 Interval = 0 means check memory once and exit
*/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

int orig (int argc, char * argv[])
{
	int interval = 5000; /* default logging every 5 seconds */
	time_t ltime;
	struct tm *today;
	MEMORYSTATUS memstat;

	if (argc > 1)
		interval = atoi(argv[1]);

	memstat.dwLength = sizeof(MEMORYSTATUS);

	do
	{
		GlobalMemoryStatus(&memstat);
		time(&ltime);
		today = localtime(&ltime);
		printf("%4.4d/%2.2d/%2.2d,%2.2d:%2.2d:%2.2d,%u,%u,%u,%u,%u,%u\n",
			today->tm_year + 1900, today->tm_mon + 1, today->tm_mday,
			today->tm_hour,	today->tm_min, today->tm_sec,
			memstat.dwAvailPhys, memstat.dwAvailPageFile, memstat.dwAvailVirtual,
			memstat.dwTotalPhys, memstat.dwTotalPageFile, memstat.dwTotalVirtual);
		fflush(stdout);
		if (interval)
			Sleep (interval);
	}
	while (interval); /* Repeat forever, if interval is non-zero  */

	return 0;
}

int main(int argc, char * argv[]) {
	FILE *fp=stdout;
    time_t ltime;
	MEMORYSTATUS memstat; 
	// orig(argc,argv); return 0;
	if (argc>1)  fp=fopen(argv[1],"a");
	
	memstat.dwLength = sizeof(MEMORYSTATUS);
	GlobalMemoryStatus(&memstat);
	time(&ltime);
	if (fp!=NULL){
	  fprintf(fp,"%lu %lu %lu\n",(unsigned int) ltime,  memstat.dwAvailPhys ,memstat.dwAvailVirtual);
	  if (fp!=stdout) fclose(fp);
	}
	//Sleep (5000);  
	return 0;
}
